@@ -1,5 +1,6 @@
 from .__version__ import __version__
 from .base import FAAPI
+from .comment import Comment
 from .journal import Journal
 from .submission import Submission
 from .submission import SubmissionPartial
@@ -9,6 +10,7 @@ from .user import UserPartial
 __all__ = [
     "__version__",
     "FAAPI",
+    "Comment",
     "Journal",
     "Submission",
     "SubmissionPartial",
