@@ -3,7 +3,7 @@ from http.cookiejar import Cookie
 from http.cookiejar import CookieJar
 from platform import python_version
 from platform import uname
-from re import sub
+from re import compile as re_compile
 from typing import Optional
 from typing import Union
 from urllib.robotparser import RobotFileParser
@@ -14,9 +14,10 @@ from requests import Response
 from requests import Session
 
 from .__version__ import __version__
+from .exceptions import Unauthorized
+from .exceptions import _raise_exception
 
 root: str = "https://www.furaffinity.net"
-user_agent: str = f"faapi/{__version__} Python/{python_version()} {(u := uname()).system}/{u.release}"
 
 
 def join_url(*url_comps: Union[str, int]) -> str:
@@ -24,8 +25,9 @@ def join_url(*url_comps: Union[str, int]) -> str:
 
 
 def make_session(cookies: Union[list[dict[str, str]], CookieJar]) -> CloudflareScraper:
+    assert len(cookies), _raise_exception(Unauthorized("No cookies for session"))
     session: CloudflareScraper = create_scraper()
-    session.headers["User-Agent"] = user_agent
+    session.headers["User-Agent"] = f"faapi/{__version__} Python/{python_version()} {(u := uname()).system}/{u.release}"
 
     for cookie in cookies:
         if isinstance(cookie, Cookie):
@@ -38,7 +40,7 @@ def make_session(cookies: Union[list[dict[str, str]], CookieJar]) -> CloudflareS
 
 def get_robots(session: Session) -> RobotFileParser:
     robots: RobotFileParser = RobotFileParser(url := join_url(root, "robots.txt"))
-    robots.parse(sub(r"\n\s*#.*\n+", "\n", session.get(url).text).splitlines())
+    robots.parse(filter(re_compile(r"^[^#].*").match, session.get(url).text.splitlines()))
     return robots
 
 
