@@ -1,16 +1,28 @@
+from collections import namedtuple
 from datetime import datetime
 from typing import Optional
 
 from .connection import join_url
 from .connection import root
+from .exceptions import _raise_exception
 from .parse import BeautifulSoup
 from .parse import Tag
 from .parse import check_page_raise
+from .parse import parse_comments
 from .parse import parse_submission_figure
 from .parse import parse_submission_page
 from .user import UserPartial
 
 
+class SubmissionStats(namedtuple("SubmissionStats", ["views", "comments", "favorites"])):
+    """
+    This object contains the submission's statistics:
+    * views
+    * comments
+    * favorites
+    """
+
+
 class SubmissionBase:
     """
     Base class for the submission objects.
@@ -21,6 +33,13 @@ class SubmissionBase:
         self.title: str = ""
         self.author: UserPartial = UserPartial()
 
+    def __eq__(self, other) -> bool:
+        if isinstance(other, SubmissionBase):
+            return other.id == self.id
+        elif isinstance(other, int):
+            return other == self.id
+        return False
+
     def __iter__(self):
         yield "id", self.id
         yield "title", self.title
@@ -44,14 +63,15 @@ class SubmissionBase:
 
 class SubmissionPartial(SubmissionBase):
     """
-    This class contains a submission's partial information.
+    Contains partial submission information gathered from submissions pages (gallery, scraps, etc.).
     """
 
     def __init__(self, submission_figure: Tag = None):
         """
         :param submission_figure: The figure tag from which to parse the submission information.
         """
-        assert submission_figure is None or isinstance(submission_figure, Tag)
+        assert submission_figure is None or isinstance(submission_figure, Tag), \
+            _raise_exception(TypeError(f"submission_figure must be {None} or {BeautifulSoup.__name__}"))
 
         super().__init__()
 
@@ -76,7 +96,9 @@ class SubmissionPartial(SubmissionBase):
 
         :param submission_figure: The optional figure tag from which to parse the submission.
         """
-        assert submission_figure is None or isinstance(submission_figure, Tag)
+        assert submission_figure is None or isinstance(submission_figure, Tag), \
+            _raise_exception(TypeError(f"submission_figure must be {None} or {BeautifulSoup.__name__}"))
+
         self.submission_figure = submission_figure or self.submission_figure
         if self.submission_figure is None:
             return
@@ -93,14 +115,15 @@ class SubmissionPartial(SubmissionBase):
 
 class Submission(SubmissionBase):
     """
-    This class contains a submission's full information.
+    Contains complete submission information gathered from submission pages, including comments.
     """
 
     def __init__(self, submission_page: BeautifulSoup = None):
         """
         :param submission_page: The page from which to parse the submission information.
         """
-        assert submission_page is None or isinstance(submission_page, BeautifulSoup)
+        assert submission_page is None or isinstance(submission_page, BeautifulSoup), \
+            _raise_exception(TypeError(f"submission_page must be {None} or {BeautifulSoup.__name__}"))
 
         super().__init__()
 
@@ -111,6 +134,7 @@ class Submission(SubmissionBase):
         self.species: str = ""
         self.gender: str = ""
         self.rating: str = ""
+        self.stats: SubmissionStats = SubmissionStats(0, 0, 0)
         self.type: str = ""
         self.description: str = ""
         self.mentions: list[str] = []
@@ -119,6 +143,8 @@ class Submission(SubmissionBase):
         self.thumbnail_url: str = ""
         self.prev: Optional[int] = None
         self.next: Optional[int] = None
+        from .comment import Comment
+        self.comments: list[Comment] = []
 
         self.parse()
 
@@ -132,6 +158,7 @@ class Submission(SubmissionBase):
         yield "species", self.species
         yield "gender", self.gender
         yield "rating", self.rating
+        yield "stats", self.stats._asdict()
         yield "type", self.type
         yield "description", self.description
         yield "mentions", self.mentions
@@ -140,6 +167,8 @@ class Submission(SubmissionBase):
         yield "thumbnail_url", self.thumbnail_url
         yield "prev", self.prev
         yield "next", self.next
+        from .comment import _remove_recursion
+        yield "comments", [dict(_remove_recursion(c)) for c in self.comments]
 
     def parse(self, submission_page: BeautifulSoup = None):
         """
@@ -147,7 +176,9 @@ class Submission(SubmissionBase):
 
         :param submission_page: The optional page from which to parse the submission.
         """
-        assert submission_page is None or isinstance(submission_page, BeautifulSoup)
+        assert submission_page is None or isinstance(submission_page, BeautifulSoup), \
+            _raise_exception(TypeError(f"submission_page must be {None} or {BeautifulSoup.__name__}"))
+
         self.submission_page = submission_page or self.submission_page
         if self.submission_page is None:
             return
@@ -167,6 +198,7 @@ class Submission(SubmissionBase):
         self.species = parsed["species"]
         self.gender = parsed["gender"]
         self.rating = parsed["rating"]
+        self.stats = SubmissionStats(parsed["views"], parsed["comment_count"], parsed["favorites"])
         self.type = parsed["type"]
         self.description = parsed["description"]
         self.mentions = parsed["mentions"]
@@ -175,3 +207,5 @@ class Submission(SubmissionBase):
         self.thumbnail_url = parsed["thumbnail_url"]
         self.prev = parsed["prev"]
         self.next = parsed["next"]
+        from .comment import sort_comments, Comment
+        self.comments = sort_comments([Comment(t, self) for t in parse_comments(self.submission_page)])
