@@ -4,6 +4,7 @@ from typing import Optional
 
 from .connection import join_url
 from .connection import root
+from .exceptions import _raise_exception
 from .parse import BeautifulSoup
 from .parse import Tag
 from .parse import check_page_raise
@@ -12,13 +13,13 @@ from .parse import parse_user_tag
 from .parse import username_url
 
 
-class UserStats(namedtuple("UserStats", ["views", "submissions", "favs", "comments_earned",
+class UserStats(namedtuple("UserStats", ["views", "submissions", "favorites", "comments_earned",
                                          "comments_made", "journals", "watched_by", "watching"])):
     """
     This object contains a user's statistics:
     * views
     * submissions
-    * favs
+    * favorites
     * comments_earned
     * comments_made
     * journals
@@ -36,6 +37,13 @@ class UserBase:
         self.name: str = ""
         self.status: str = ""
 
+    def __eq__(self, other) -> bool:
+        if isinstance(other, UserBase):
+            return other.name_url == self.name_url
+        elif isinstance(other, str):
+            return username_url(other) == self.name_url
+        return False
+
     def __iter__(self):
         yield "name", self.name
         yield "status", self.status
@@ -53,7 +61,7 @@ class UserBase:
 
         :return: The cleaned username.
         """
-        return username_url(self.name.lower())
+        return username_url(self.name)
 
     @property
     def url(self):
@@ -67,14 +75,15 @@ class UserBase:
 
 class UserPartial(UserBase):
     """
-    This class contains a user's partial information.
+    Contains partial user information gathered from user folders (gallery, journals, etc.) and submission/journal pages.
     """
 
     def __init__(self, user_tag: Tag = None):
         """
         :param user_tag: The tag from which to parse the user information.
         """
-        assert user_tag is None or isinstance(user_tag, Tag)
+        assert user_tag is None or isinstance(user_tag, Tag), \
+            _raise_exception(TypeError(f"user_tag must be {None} or {Tag.__name__}"))
 
         super().__init__()
 
@@ -98,7 +107,9 @@ class UserPartial(UserBase):
 
         :param user_tag: The tag from which to parse the user information.
         """
-        assert user_tag is None or isinstance(user_tag, Tag)
+        assert user_tag is None or isinstance(user_tag, Tag), \
+            _raise_exception(TypeError(f"user_tag must be {None} or {Tag.__name__}"))
+
         self.user_tag = user_tag or self.user_tag
         if self.user_tag is None:
             return
@@ -113,14 +124,15 @@ class UserPartial(UserBase):
 
 class User(UserBase):
     """
-    This class contains a user's full information.
+    Contains complete user information gathered from userpages.
     """
 
     def __init__(self, user_page: BeautifulSoup = None):
         """
         :param user_page: The page from which to parse the user information.
         """
-        assert user_page is None or isinstance(user_page, BeautifulSoup)
+        assert user_page is None or isinstance(user_page, BeautifulSoup), \
+            _raise_exception(TypeError(f"user_page must be {None} or {BeautifulSoup.__name__}"))
 
         super().__init__()
 
@@ -152,7 +164,9 @@ class User(UserBase):
 
         :param user_page: The page from which to parse the user information.
         """
-        assert user_page is None or isinstance(user_page, BeautifulSoup)
+        assert user_page is None or isinstance(user_page, BeautifulSoup), \
+            _raise_exception(TypeError(f"user_page must be {None} or {BeautifulSoup.__name__}"))
+
         self.user_page = user_page or self.user_page
         if self.user_page is None:
             return
